/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.notifier;

import java.io.*;
import cgl.narada.event.NBEvent;
import cgl.util.BootStrap;
import java.util.Properties;



import org.apache.axis.types.URI;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.SessionService;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.EventProducer;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import java.util.Random;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.apache.axiom.om.OMElement;

import cgl.fthpis.datatype.attribute.Attribute;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMNamespace;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class ContextPublisher {

    private Object synchObj = null;
    private Properties bootstrap = null;
    private String hostName = null;
    private String portNum = null;
    private ClientService clientService = null;
    private int entityId;
    private String serviceConfigPath;
    private EventProducer publisher = null;
    private static ContextPublisher context_publisher = null;
    private Random generator = null;
    private static Log log = LogFactory.getLog(ContextPublisher.class);

    public static ContextPublisher getInstance(Properties props) {
        if (context_publisher == null) {
            context_publisher = new ContextPublisher(props);
        }
        return context_publisher;
    }

    public ContextPublisher(Properties props) {
        try {
            this.bootstrap = props;
            this.hostName = bootstrap.getProperty("hostname");
            this.portNum = bootstrap.getProperty("portnum");
            this.serviceConfigPath = "properties.properties";
            generator = new Random(System.currentTimeMillis());
            entityId = generator.nextInt(1000000);
            String endpoint = "niotcp://" + this.hostName + ":" + this.portNum;
            log.info("endpoint ----> " + endpoint);
            log.info("entityId ----> " + entityId);
            log.info("serviceconfigpath ----> " + this.serviceConfigPath);

            initBroker(props);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void initBroker(Properties bootstrap) {
//        try {
//            SessionService.setServiceConfigurationLocation("properties.properties");
//            clientService = SessionService.getClientService(entityId);
//        } catch (Exception ex) {
//            ex.printStackTrace();
//        }
        try {
            SessionService.setServiceConfigurationLocation("properties.properties");
            clientService = SessionService.getClientService(entityId);
        } catch (Exception ex) {
            try {
                SessionService.setServiceConfigurationLocation("/usr/local/wscontext/properties.properties");
                clientService = SessionService.getClientService(entityId);
            } catch (Exception e) {
                try {
                    SessionService.setServiceConfigurationLocation("/tmp/wscontext/properties.properties");
                    clientService = SessionService.getClientService(entityId);
                } catch (Exception ex1) {
                    System.out.println("Error in loading Property file. " + e.getMessage());
                    System.exit(1);

                }
            }
        }
        try {
            //bootstrap.put("hostname", "localhost");
            //bootstrap.put("portnum", "3045");
            clientService.initializeBrokerCommunications(bootstrap, "niotcp");
            publisher = clientService.createEventProducer();

            System.out.println("PUBLISHER CREATES LINK TO BROKER");

            // initializeTransmitter();
            // connector.initializeReceiver();
        } catch (ServiceException ex) {
            //ex.printStackTrace();

       }
    }

    public void onEvent(NBEvent nbEvent) {
        log.info("NBEvent received:");
//        synchronized (synchObj) {
//            ContextDetail context_detail = null;
//            context_detail = (ContextDetail)this.deserializeObject(nbEvent.getContentPayload());
//            printContextDetail(context_detail);
//        }
    }
/*
    public static void printContext(Context context) {
                    if (context != null) {
//                        log.info(
//                                "********* PRINT Context ***********");
//                        log.info("saved context_key = " +
//                                           context.getContextKey());
//                        log.info("context operation type = " + context.getValueType());
//                        log.info("context DATA = " + context.getValue());
//                        Name[] returned_name_array = context.getName();
//                        if (returned_name_array != null &&
//                            returned_name_array.length != 0) {
//                            log.info("context user identifer = " +
//                                               returned_name_array[0].getValue());
//                        }
//
//
//                       log.info("context session key = " + context.getSessionKey());

                       System.out.println("********* PRINT Context ***********");
                       System.out.println("saved context_key = " + context.getContextKey());
//                       System.out.println("context operation type = " + context.getValueType());
                       System.out.println("context session key = " + context.getSessionKey());
                       System.out.println("context DATA = " + context.getValue());
                       System.out.println("context version = " + context.getVersion());
                     }
    }
    */
/*
    public static void printContextDetail(ContextDetail detail) {
        if (detail != null) {
            ContextBag contextBag = detail.getContextBag();
            if (contextBag != null &&
                contextBag.getContext() != null &&
                contextBag.getContext().length != 0) {
                Context[] context_arr = contextBag.getContext();
                for (int i = 0; i < context_arr.length; i++) {
                    Context context = context_arr[i];
                    if (context != null) {
                        log.info("********* PRINT Context ***********");
                        log.info("saved context_key = " + context.getContextKey());
                        System.out.println("********* PRINT Context ***********");
                        System.out.println("saved context_key = " + context.getContextKey());

//                        //create a factory
//                        OMFactory factory = OMAbstractFactory.getOMFactory();
////use the factory to create two namespace objects
//                        OMNamespace ns1 = factory.createOMNamespace("bar","x");
//                        OMNamespace ns2 = factory.createOMNamespace("bar1","y");
////use the factory to create three elements
//
//                        OMElement root = factory.createOMElement("root", "x");
//                        root.setText("hello");

                        OMFactory factory = OMAbstractFactory.getOMFactory();
                        OMNamespace ns1 = factory.createOMNamespace("ns","x");
                        OMElement root = factory.createOMElement("root",ns1);

                        root.setLocalName("name");

                        System.out.println("name = " +  root.getLocalName());

//                        //OMNamespace ns2 = root.declareNamespace("bar1","y");
//                        OMElement elt1 = factory.createOMElement("foo",ns1);
//                        OMElement elt2 = factory.createOMElement("yuck",ns2);
//                        OMText txt1 = factory.createOMText(elt2,"blah");
//                        elt2.addChild(txt1);
//                        elt1.addChild(elt2);
//                        root.addChild(elt1);


//
//                        OMElement elt11 = factory.createOMElement("foo1",ns1);
//                        OMElement elt12 = factory.createOMElement("foo2",ns1);

                        //String contextID = context.getName();

//                        Name[] returned_name_array = context.getName();
//                        if (returned_name_array != null && returned_name_array.length != 0) {
//                            log.info("context user identifer = " + returned_name_array[0].getValue());
//                        }
                        log.info("context session key = " + context.getSessionKey());
                        log.info("context DATA = " + context.getValue());
                        System.out.println("context session key = " + context.getSessionKey());
                        System.out.println("context DATA = " + context.getValue());
                     }
                }
            }
        }
    }
*/
    /**
     * This function is to print context object to Console.
     */
    public static void printServiceAttribute(Attribute attribute) {
        if (attribute != null) {
            System.out.println(
                    "********* PRINT ServiceAttribute ***********");
            System.out.println("saved attributeKey = " + attribute.getAttributeKey());
            System.out.println("attribute attributeID = " + attribute.getAttributeName());//.getAttributeName());
            System.out.println("attribute service key = " + attribute.getServiceKey());
            System.out.println("attribute DATA = " + attribute.getAttributeValue());//.getAttributeValue());
        }
    }

    protected Object deserializeObject(byte[] bytes) {
        Object obj = null;
        try {
            ObjectInputStream in = new ObjectInputStream(new
                    ByteArrayInputStream(
                            bytes));
            obj = in.readObject();
            in.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return obj;
    }

    protected byte[] serializeObject(Object object) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            ObjectOutput out = new ObjectOutputStream(bos);
            out.writeObject(object);
            out.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // Get the bytes of the serialized object
        return bos.toByteArray();
    }

    public void publishMessage (String topic, Attribute att) {
       try {
           log.info("publisher "      + publisher);
           log.info("attribute "      + att);
           log.info("topic = "        + topic);
           NBEvent nbEvent =
                   publisher.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                           topic,
                                           serializeObject(att));
           publisher.publishEvent(nbEvent);
       } catch (Exception ex) {
           ex.printStackTrace();
       }
    }


    public void publishMessage (String topic, byte [] arr) {
       try {
           log.info("publisher "      + publisher);
           log.info("topic = "        + topic);
           NBEvent nbEvent =
                   publisher.generateEvent(TemplateProfileAndSynopsisTypes.STRING,
                                           topic,
                                           arr);
           System.out.println("PUBLISHER PUBLISHES TO TOPIC => " + topic);
           publisher.publishEvent(nbEvent);
       } catch (Exception ex) {
           ex.printStackTrace();
       }
    }

    public void close() {
    try {
        clientService.closeBrokerConnection();
        clientService.terminateServices();
    } catch (ServiceException ex) {
    }
    }

    public static void main(String[] args) {
        Properties props = new Properties();
        try {
            props = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
            log.info("Error in loading Property file. " + ex.getMessage());
            System.exit(1);
        }
        ContextPublisher cp = ContextPublisher.getInstance(props);

        cgl.fthpis.datatype.attribute.Attribute att =
            new cgl.fthpis.datatype.attribute.Attribute();
         //system identifier
        att.setAttributeKey("uuid:key");
        //user identifier
        cgl.fthpis.datatype.attribute.AttributeName att_name =
            new cgl.fthpis.datatype.attribute.AttributeName();
        att_name.setAttributeName("some name");
        //att.setAttributeName(att_name);
        //session key
        att.setSessionKey("uuid:key");
//        //value
//        cgl.fthpis.datatype.attribute.AttributeValue att_value =
//            new cgl.fthpis.datatype.attribute.AttributeValue();
//        att_value.setAttributeValue("some value".getBytes());
//        att.setAttributeValue(att_value);
//        //value type
////        cgl.fthpis.datatype.attribute.AttributeValueType att_valuetype =
////            new cgl.fthpis.datatype.attribute.AttributeValueType();
////        att_valuetype.setAttributeValueType(object.getValueType());
////        att.setAttributeValueType(att_valuetype);
//        //lease
//        cgl.fthpis.datatype.lease.Lease lease =
//            new cgl.fthpis.datatype.lease.Lease();
//        lease.setIsInfinite("true");
//        att.setLease(lease);
//
////        ContextDetail context_detail = new ContextDetail();
////        ContextBag contextBag = new ContextBag();
////        Context [] context_arr = new Context[1];
////        Context context = new Context();
////        URI system_uri = null;
////        try {
////            system_uri = new URI("context://systemuri/dummy");
////        } catch (Exception ex) {
////        }
////
////        Lease lease = new Lease();
////        lease.setIsInfinite(true);
////        lease.setTimeout("100");
////        lease.setLeaseStartTimeStamp("start_time");
////        lease.setLeaseExpirationTimeStamp("stop_time");
////
////        AccessRightInfo info = new AccessRightInfo();
////        Group group = new Group();
////        group.setReadAccess(true);
////        group.setWriteAccess(true);
////        info.setGroup(group);
////
////
////        context.setContextKey(null);
////        context.setSessionKey(null);
////        context.setServiceKey(null);
////        context.setName(null); //make this as user defined identifier instead of name
////        //context.setFindQualifier("");
////        context.setLease(lease);
////        context.setAccessRightInfo(info);
////        context.setValue("VALUE");
//////        context.setValueType("VALUETYPE");
////        context.setVersion(123);
////
////        context_arr[0] = context;
////        contextBag.setContext(context_arr);
////        context_detail.setContextBag(contextBag);
//
//        //printContext(context);
//        printServiceAttribute(att);
//        //printContextDetail(context_detail);
//
        cp.publishMessage("/session://dummytopic", att);
//
        cp.close();
//        log.info("hello");

        OMFactory factory = OMAbstractFactory.getOMFactory();
        OMNamespace ns1 = factory.createOMNamespace("ns","x");
        OMElement root = factory.createOMElement("root",ns1);

        root.setLocalName("name");

        System.out.println("name = " +  root.getLocalName());
    }
}
